package com.nisum.task.controller;

import com.nisum.task.entity.BPN;
import com.nisum.task.service.BPNService;
import com.nisum.task.service.FileUploadService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

import java.io.IOException;

@CrossOrigin(origins = "*")
@Slf4j
@RestController
@RequestMapping("/bpn")
public class BPNController {

    @Autowired
    private BPNService bPNService;

    @Autowired
    private FileUploadService fileUploadService;

    @PostMapping("/createBPN")
    public Mono<BPN> saveBPN(@RequestBody BPN bpn) {
        log.info("saveBPN {}",bpn);
        return bPNService.saveBPN(bpn);
    }

    @GetMapping("/findById/{bpnID}")
    public Mono<BPN> getBPN(@PathVariable String bpnID) {
        log.info("getBPN {}",bpnID);
        return bPNService.findById(Long.valueOf(bpnID));
    }

    @GetMapping("/findAll")
    public Flux<BPN> findAllBpns() {
        log.info("find all");
        return bPNService.findAll();
    }


    @PostMapping("/processBPN")
    public Mono<BPN> processBPN(@RequestBody BPN bpn) {
        log.info("processBPN {}",bpn);
        return bPNService.processBPN(bpn);
    }

    //@PostMapping(value  ="/uploadFile",consumes="multipart/form-data")
//    @RequestMapping(value = "/upload", method = RequestMethod.PUT) // Or POST
//    @ResponseStatus(HttpStatus.OK)
//    public Flux<BPN> uploadFile(@RequestParam("file") MultipartFile file) throws IOException {
//        log.info("file upload");
//        return fileUploadService.uploadFile(file);
//    }

    @RequestMapping(value = "/upload", method = RequestMethod.PUT,consumes="multipart/form-data")
    @ResponseStatus(HttpStatus.OK)
    public Flux<BPN> uploadFile(@RequestParam("file") MultipartFile file) throws IOException {
        log.info("file upload");

        // Log details of the file
        log.info("File Name: {}", file.getOriginalFilename());
        log.info("Content Type: {}", file.getContentType());
        log.info("File Size: {}", file.getSize());

        return fileUploadService.uploadFile(file);
    }


    @PostMapping("/uploadFile")
    public ResponseEntity<BPN> uploadFiles(@RequestParam("file") MultipartFile file) {
        String message = "";
        try {
            fileUploadService.save(file);

            message = "Uploaded the file successfully: " + file.getOriginalFilename();
            return ResponseEntity.status(HttpStatus.OK).body(new BPN());
        } catch (Exception e) {
            message = "Could not upload the file: " + file.getOriginalFilename() + ". Error: " + e.getMessage();
            return ResponseEntity.status(HttpStatus.EXPECTATION_FAILED).body(new BPN());
        }
    }



}
