package com.nisum.task.service;

import com.nisum.task.entity.ActionEnum;
import com.nisum.task.entity.BPN;
import com.nisum.task.repository.BPNRepository;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

import java.util.Date;

@Service
@Slf4j
public class BPNService {

    private static final String DELETE = "Deleted";
    private static final String BLOCK = "block";
    private static final String FORCE = "force";
    private static final String RECALL = "recall";
    private static final String ACTIVE = "Active";
    private static final String IN_ACTIVE = "In Active";

    @Autowired
    private BPNRepository bpnRepository;

    public Mono<BPN> saveBPN(BPN bpn) {
        bpn.setFirstEffectiveDate(new Date());
        bpn.setCreatedTime(new Date());
        bpn.setUpdatedTime(new Date());
        log.info("bpn {} ", bpn);
        return processBPN(bpn).flatMap(bpn1 -> bpnRepository.save(bpn1));
    }

    public Mono<BPN> findById(Long bpnId) {
        return bpnRepository.findById(bpnId);
    }

    public Flux<BPN> findAll() {
        return bpnRepository.findAll();
    }

    public Mono<BPN> processBPN(BPN bpnObject) {

        return Mono.just(bpnObject).flatMap(bpn2 -> {
            if (bpn2.getForceOptCd().equalsIgnoreCase(ActionEnum.BLOCK.name())) {
                return bpnRepository.findByBpn(bpn2.getBpn())
                        .flatMap(bpn1 -> Mono.just(setAllFields(bpn1, bpn2, "BPN Blocked successfully", bpn2.getForceOptCd())))
                        .switchIfEmpty(Mono.defer(() -> {
                            bpn2.setBpnProcessMessage("Blocked Successfully");
                            bpn2.setBpnStatus(bpn2.getForceOptCd());
                            return Mono.just(bpn2);
                        }));
            } else if (bpn2.getForceOptCd().equalsIgnoreCase(ActionEnum.FORCE.name())) {
                return bpnRepository.findByBpn(bpn2.getBpn())
                        .flatMap(bpn1 -> Mono.just(setAllFields(bpn1, bpn2, "BPN Forced successfully", bpn2.getForceOptCd())))
                        .switchIfEmpty(Mono.defer(() -> {
                            bpn2.setBpnProcessMessage("BPN Forced successfully");
                            bpn2.setBpnStatus(bpn2.getForceOptCd());
                            return Mono.just(bpn2);
                        }));
            } else if (bpn2.getForceOptCd().equalsIgnoreCase(ActionEnum.RECALL.name())) {
                return bpnRepository.findByBpn(bpn2.getBpn()).
                        flatMap(bpn -> Mono.just(setAllFields(bpn, bpn2, "BPN Recalled successfully", bpn2.getForceOptCd())))
                        .switchIfEmpty(Mono.defer(() -> {
                            bpn2.setBpnProcessMessage("BPN Recalled successfully");
                            bpn2.setBpnStatus(bpn2.getForceOptCd());
                            return Mono.just(bpn2);
                        }));
            } else {
                return Mono.just(bpn2);
            }
        });
    }

    private BPN setAllFields(BPN bpnDB, BPN uiBPN, String message, String status) {
        bpnDB.setBpnStatus(status);
        bpnDB.setLasttEffectiveDate(uiBPN.getLasttEffectiveDate());
        bpnDB.setUpdatedTime(new Date());
        bpnDB.setFac(uiBPN.getFac());
        bpnDB.setForceOptCd(uiBPN.getForceOptCd());
        bpnDB.setBpnProcessMessage(message);
        log.info("bpnDB {}", bpnDB);
        return bpnDB;
    }
}
