package com.nisum.task.service;

import com.nisum.task.entity.BPN;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.xssf.usermodel.XSSFRow;
import org.apache.poi.xssf.usermodel.XSSFSheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;
import reactor.core.publisher.Flux;

import java.io.IOException;
import java.nio.file.FileAlreadyExistsException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.List;

@Service
@Slf4j
public class FileUploadService {

    @Autowired
    private BPNService bpnService;
    private final Path root = Paths.get("uploads");


    public Flux<BPN> uploadFile(MultipartFile file) throws IOException {

        List<BPN> tempBPNList = new ArrayList<>();
        XSSFWorkbook workbook = new XSSFWorkbook(file.getInputStream());
        XSSFSheet worksheet = workbook.getSheetAt(0);

        for (int i = 1; i < worksheet.getPhysicalNumberOfRows(); i++) {
            BPN tempBPN = new BPN();

            XSSFRow row = worksheet.getRow(i);
            tempBPN.setBpn(String.valueOf(row.getCell(0)));
            tempBPN.setForceOptCd(String.valueOf(row.getCell(1)));
            tempBPNList.add(tempBPN);
        }
        log.info("Data {}",tempBPNList);
        return Flux.empty();
    }

    public void save(MultipartFile file) {
        try {
            Files.createDirectories(root);

            Files.copy(file.getInputStream(), this.root.resolve(file.getOriginalFilename()));
        } catch (Exception e) {
            if (e instanceof FileAlreadyExistsException) {
                throw new RuntimeException("A file of that name already exists.");
            }

            throw new RuntimeException(e.getMessage());
        }
    }


}
